@@ -64,9 +64,7 @@ on_open_prefs_activated (GSimpleAction *action,
 {
   MsPluginRow *self = MS_PLUGIN_ROW (data);
 
-  gtk_widget_activate_action (GTK_WIDGET (self),
-                              "plugin-list-box.open-plugin-prefs",
-                              "s", self->filename);
+  ms_plugin_row_open_prefs (self);
 }
 
 
@@ -413,3 +411,18 @@ ms_plugin_row_get_enabled (MsPluginRow *self)
 
   return self->enabled;
 }
+
+
+void
+ms_plugin_row_open_prefs (MsPluginRow *self)
+{
+  g_assert (MS_IS_PLUGIN_ROW (self));
+
+  if (!self->has_prefs)
+    return;
+
+  gtk_widget_activate_action (GTK_WIDGET (self),
+                              "plugin-list-box.open-plugin-prefs",
+                              "s",
+                              self->filename);
+}
